package markov;

import java.util.Base64;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Shortener {

    private final Data data;

    public Shortener(Data data) {
        this.data = data;
    }

    public String getId(Sentence sentence) {
        return fromInts(sentence.asStream()
                .flatMap(d -> {
                    if (d.getLookup().getDistinctTokens() == 1) {
                        return Stream.empty();
                    } else
                        return Stream.of(d.getId());
                }));

    }

    String fromInts(Stream<Integer> ints) {
        String cps = ints.map(i -> new String(Character.toChars(i))).collect(Collectors.joining());
        return Base64.getEncoder().encodeToString(cps.getBytes());
    }

    Stream<Integer> toInts(String hash) {
        return new String(Base64.getDecoder().decode(hash.getBytes())).codePoints().boxed();
    }

    public Sentence getSentence(String hash) {
        Integer[] ids = toInts(hash).toArray(Integer[]::new);

        Sentence sentence = new Sentence(data);
        Prefix prefix = new Prefix(new Token[0]);
        int i = 0;
        while (i < ids.length) {
            Lookup lookup = data.fetch(prefix);
            Decission d = lookup.getDistinctTokens() == 1 ? lookup.average() : lookup.forId(ids[i++]);
            d.setP(prefix);
            d.setLookup(lookup);
            sentence.add(d);
            prefix = prefix.slide(d.getToken(), data.getPrefixLength());
        }
        return sentence;
    }
}
